using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Grouping;

namespace _2942
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            
			DataTable parentTable = GetParentTable();
			DataTable childTable = GetChildTable();

			DataSet ds = new DataSet();
			ds.Tables.AddRange(new DataTable[]{parentTable, childTable});

			DataRelation parentToChild = new DataRelation("ParentToChild", parentTable.Columns["parentID"], childTable.Columns["ParentID"]);
			ds.Relations.Add(parentToChild);
            this.gridGroupingControl1.DataSource = ds.Tables[0];

            this.gridGroupingControl1.AllowDrop = true;

            this.gridGroupingControl1.TableControlMouseDown += new GridTableControlMouseEventHandler(gridGroupingControl1_TableControlMouseDown);
            this.gridGroupingControl1.DragOver += new DragEventHandler(gridGroupingControl1_DragOver);
            this.gridGroupingControl1.DragDrop += new DragEventHandler(gridGroupingControl1_DragDrop);
            this.gridGroupingControl1.DragLeave += new EventHandler(gridGroupingControl1_DragLeave);
            this.gridGroupingControl1.DragEnter += new DragEventHandler(gridGroupingControl1_DragEnter);
            this.gridGroupingControl1.TableControlPrepareViewStyleInfo += new GridTableControlPrepareViewStyleInfoEventHandler(gridGroupingControl1_TableControlPrepareViewStyleInfo);

        }

        void gridGroupingControl1_TableControlPrepareViewStyleInfo(object sender, GridTableControlPrepareViewStyleInfoEventArgs e)
        {
            Rectangle rect = e.TableControl.RangeInfoToRectangle(GridRangeInfo.Cell(e.Inner.RowIndex,e.Inner.ColIndex));
            int row, col;
            this.gridGroupingControl1.TableControl.PointToRowCol(new Point(rect.X, rect.Y), out row, out col);
            GridTableCellStyleInfo style = e.TableControl.PointToTableCellStyle(new Point(rect.X,rect.Y));

            if (row == targetRow && (style.TableCellIdentity.TableCellType == GridTableCellType.RecordFieldCell ||
                style.TableCellIdentity.TableCellType == GridTableCellType.AlternateRecordFieldCell))
                e.Inner.Style.BackColor = Color.Red;
        }

        void gridGroupingControl1_DragEnter(object sender, DragEventArgs e)
        {
            targetRow = -1;
            targetCol = -1;
            e.Effect = DragDropEffects.All;
        }

        void gridGroupingControl1_DragLeave(object sender, EventArgs e)
        {
            int oldRow = targetRow;
            int oldCol = targetCol;
            targetRow = -1;
            targetCol = -1;
            this.gridGroupingControl1.TableControl.RefreshRange(GridRangeInfo.Row(oldRow));
        }

        void gridGroupingControl1_DragDrop(object sender, DragEventArgs e)
        {

            GridGroupingControl grid = sender as GridGroupingControl;

            int oldRow = targetRow;
            int oldCol = targetCol;
            targetRow = -1;
            targetCol = -1;
            grid.TableControl.RefreshRange(GridRangeInfo.Row(oldRow));

            GridTableCellStyleInfo style = grid.TableControl.PointToTableCellStyle(grid.TableControl.PointToClient(Control.MousePosition));
            if (style.TableCellIdentity.DisplayElement is GridRecordRow)
            {
                GridRecordRow recRow = style.TableCellIdentity.DisplayElement as GridRecordRow;
                grid.Table.SelectedRecords.Clear();
                recRow.GetRecord().SetCurrent();
                string recStr = "";
                DataRowView drv = recRow.GetData() as DataRowView;
                int i = 0;
                while (i < drv.Row.ItemArray.Length)
                {
                    recStr += drv[i].ToString();
                    if(++i<drv.Row.ItemArray.Length)
                        recStr += ", ";
                }
                MessageBox.Show(recStr, "Record Values");
                
                if (recRow.ParentChildTable.ParentNestedTable != null)
                {
                    Record rec = recRow.ParentChildTable.ParentNestedTable.ParentRecord;
                    drv = rec.GetData() as DataRowView;
                    i = 0;
                    recStr = "";
                    while (i < drv.Row.ItemArray.Length)
                    {
                        recStr += drv[i].ToString();
                        if (++i < drv.Row.ItemArray.Length)
                            recStr += ", ";
                    }
                    MessageBox.Show(recStr, "Parent Record Values");
                }
                
            }
        }


        void gridGroupingControl1_DragOver(object sender, DragEventArgs e)
        {

            GridGroupingControl grid = sender as GridGroupingControl;

            int row = -1;
            int col = -1;
            Point pt = grid.PointToClient(new Point(e.X, e.Y));
            
            if (grid.TableControl.PointToRowCol(pt, out row, out col))
            {
                int oldRow = targetRow;
                int oldCol = targetCol;
                targetRow = row;
                targetCol = col;
                grid.TableControl.RefreshRange(GridRangeInfo.Row(targetRow));
                grid.TableControl.RefreshRange(GridRangeInfo.Row(oldRow));
            }

            // showing the dragdrop effects.
            if (e.Data.GetDataPresent(typeof(SelectedRecordsCollection)))
            {
                e.Effect = DragDropEffects.Copy;
            }
        }

        void gridGroupingControl1_TableControlMouseDown(object sender, GridTableControlMouseEventArgs e)
        {
            
            GridGroupingControl grid = sender as GridGroupingControl;
            if (grid != null)
            {
                GridTableControl tableControl = grid.TableControl;
                Point pt = new Point(e.Inner.X, e.Inner.Y);
                int row, col;
                if (tableControl.PointToRowCol(pt, out row, out col))
                {
                    GridTableCellStyleInfo style = tableControl.PointToTableCellStyle(pt);
                    //checking if the cell in which the MouseDown event is a RecordRowHeaderCell.
                    //If yes, then begins the DragDrop operation.

                    //Console.WriteLine(style.TableCellIdentity.TableCellType.ToString());
                    if ((style.TableCellIdentity.TableCellType == GridTableCellType.RecordRowHeaderCell
                        || style.TableCellIdentity.TableCellType == GridTableCellType.AlternateRecordRowHeaderCell)
                        && style.TableCellIdentity.DisplayElement.ParentRecord != null
                        && style.TableCellIdentity.DisplayElement.ParentRecord.IsSelected())
                    {


                        Element el = grid.Table.DisplayElements[row];
                        DragDropEffects ef;
                        if (el.Kind == DisplayElementKind.NestedTable)
                        {
                            GridNestedTable nt = (GridNestedTable)el;
                            Console.WriteLine(nt.ChildTable.Name.ToString());
                            Console.WriteLine(grid.GetTable(nt.ChildTable.Name.ToString()).SelectedRecords[0].Record.ToString());
                            ef = tableControl.DoDragDrop(new DataObject(grid.GetTable(nt.ChildTable.Name.ToString()).SelectedRecords), DragDropEffects.All);
                        }
                        else
                            ef = tableControl.DoDragDrop(new DataObject(grid.Table.SelectedRecords), DragDropEffects.All);

                        if (ef == DragDropEffects.Move)
                        {
                            //need to delete selection if you want to support this
                        }
                    }
                }
            }
        }


#region Data Tables
        private int numberParentRows = 10;
        private int numberChildRows = 20;
        private DataTable GetParentTable()
        {
            DataTable dt = new DataTable("ParentTable");


            dt.Columns.Add(new DataColumn("parentID")); //lower case p
            dt.Columns.Add(new DataColumn("SomethingEXIST"));
            dt.Columns.Add(new DataColumn("Checked", typeof(bool)));

            Random ran = new Random();
            for (int i = 0; i < numberParentRows; ++i)
            {
                DataRow dr = dt.NewRow();
                dr[0] = i;
                dr[1] = ran.Next(100) % 2;
                dr[2] = true;
                dt.Rows.Add(dr);
            }

            return dt;
        }

        private DataTable GetChildTable()
        {
            DataTable dt = new DataTable("ChildTable");

            dt.Columns.Add(new DataColumn("childID")); //lower case c
            dt.Columns.Add(new DataColumn("ParentID")); //upper case P
            dt.Columns.Add(new DataColumn("SomethingEXIST"));
            dt.Columns.Add(new DataColumn("Checked", typeof(bool)));
            Random ran = new Random();
            for (int i = 0; i < numberChildRows; ++i)
            {
                DataRow dr = dt.NewRow();
                dr[0] = i.ToString();
                dr[1] = (i % numberParentRows).ToString();
                dr[2] = ran.Next(100) % 2; ;
                dr[3] = true;
                dt.Rows.Add(dr);
            }

            return dt;
        }

        #endregion

        int targetRow = -1;
        int targetCol = -1;

        private void label1_MouseDown(object sender, MouseEventArgs e)
        {
            this.label1.DoDragDrop("", DragDropEffects.All);
        }


    }
}